Title: Backup and Recovery - OpenStack
Status: In Progress

# Backup and Recovery - OpenStack

## Introduction

The OpenStack flexibility makes backup and restore to a very individual process
depending on the used components. This section describes how the critical parts
like the configuration files and databases OpenStack needs to run are saved. As
before for Juju it doesn't describe ho to back up the objects inside the Object 
Storage or the data inside the Block Storage.

## Scope

**TODO**

## Backup Cloud Controller Database

Like Juju the OpenStack cloud controller uses a database server which stores the
central databases for Nova, Glance, Keystone, Cinder, and Switft. You can backup
the five databases into one common dump:

````
$ mysqldump --opt --all-databases > openstack.sql
````

Alternatively you can backup the database for each component individually:

````
$ mysqldump --opt nova > nova.sql
$ mysqldump --opt glance > glance.sql
$ mysqldump --opt keystone > keystone.sql
$ mysqldump --opt cinder > cinder.sql
$ mysqldump --opt swift > swift.sql
````

## Backup File Systems

Beside the databases OpenStack uses different directories for its configuration,
runtime files, and logging. Like the databases they are grouped individually per
component. This way also the backup can be done per component.

### Nova

You'll find the configuration directory `/etc/nova` on the cloud controller and 
each compute node. It should be regularly backed up.

Another directory to backup is `/var/lib/nova`. But here you have to be careful
with the `instances` subdirectory on the compute nodes. It contains the KVM images
of the running instances. If you want to maintain backup copies of those instances
you can do a backup here too. In this case make sure to not save a live KVM instance
because it may not boot properly after restoring the backup.

Third directory for the compute component is `/var/log/nova`. In case of a central
logging server this directory does not need to be backed up. So we suggest you to
run your environment with this kind of logging.

### Glance

Like for Nova you'll find the directories `/etc/glance` and `/var/log/glance`, the
handling should be the same here too.

Glance also uses the directory named `/var/lib/glance` which also should be backed
up.

### Keystone

Keystone is using the directories `/etc/keystone`, `/var/lib/keystone`, and
`/var/log/keystone`. They follow the same rules as Nova and Glance. Even if
the `lib` directory should not contain any data being used, can also be backed 
up just in case.

### Cinder

Like before you'll find the directories `/etc/cinder`, `/var/log/cinder`,
and `/var/lib/cinder`. And also here the handling should be the same. Opposite
to Nova abd Glance there's no special handling of `/var/lib/cinder` needed.

### Swift

Beside the Swift configuration the directory `/etc/swift` contains the ring files
and the ring builder files. If those get lest the data on your data gets inaccessable.
So you can easily imagine how important it is to backup this directory. Best practise
is to copy the builder files to the storage nodes along with the ring files. So
multiple copies are spread throughout the cluster.

**TODO(mue)** Really needed when we use Ceph for storage?

## Restore

The restore based on the backups is a step-by-step process restoring the components
databases and all their directories. It's important that the component to restore is 
currently not running. So always start the restoring with stopping all components.

Let's take Nova as an example. First execute

````
$ stop nova-api
$ stop nova-cert
$ stop nova-consoleauth
$ stop nova-novncproxy
$ stop nova-objectstore
$ stop nova-scheduler
````

on the cloud controller to savely stop the processes of the component. Next step is the
restore of the database. By using the `--opt` option during backup we ensured that all
tables are initially dropped and there's no conflict with currently existing data in
the databases.

````
$ mysql nova < nova.sql
````

Before restoring the directories you should move at least the configuration directoy,
here `/etc/nova`, into a secure location in case you need to roll it back.

After the database and the files are restored you can start MySQL and Nova again.

````
$ start mysql
$ start nova-api
$ start nova-cert
$ start nova-consoleauth
$ start nova-novncproxy
$ start nova-objectstore
$ start nova-scheduler
````

The process for the other components look similar.
